/*
 *  main.h
 *  
 *
 *  Created by Alexander Thomas on 2006/07/21.
 *  Last modified on 2011/10/29
 *
 */

typedef struct
{
	bool  bClean;
	bool  bFast;
	bool  bForce;
	bool  bCompress;
	bool  bReplaceV1;
	bool  bDropV1;
	bool  bNukem;
	int   nAlbumMode;
	bool  bSoundCheck;    // old -i mode
	bool  bSoundCheckTag; // new -I mode
	bool  bSCheckAlbum;   // use album gain for -i, -I
	bool  bKillSCheck;
	float fAGain;
	unsigned long wXPadding;
	unsigned long wMinTagSize;
} Settings;

enum albumMode_t {NOALBUM, ALBUM_AUTO, ALBUM_USER};


void process_arguments(int argc, char **argv, vector<char*> &vsFiles,
                       Settings &settings);

void error_exit(const char *msg, int val, const char *pName);

void getID3Info(const char* filePath,
                long &id3_length, bool &hasV1, bool &hasV2);

/* Convert the dB value of dGain and the [0,1] value of dPeak to values
 * as used in the iTunNORM tag. */
void iTunizeGains(double dGain, double dPeak,
                  unsigned int &nGain1000, unsigned int &nGain2500,
                  unsigned int &nPeak);

/* Retrieve ReplayGain tag values if they exist. If not, the returned peak values
 * will be negative. If gain values were found but no peak values, peak values
 * will be returned as 0.0. */
bool getReplayTags(struct id3_tag *fileTag,
                   double &dGain, double &dPeak, double &dAGain, double &dAPeak);

/* Return the contents of the iTunNORM field if it exists, empty string otherwise. */
string getITunNORM(struct id3_tag *fileTag);

/* Try to port the precalculated ReplayGain values in each file's tag
 * to existing iTunNORM fields in the tags. */
void fixSoundCheck(const vector<char*> &theFiles, bool bAlbum);

/*--- File routines ---*/
unsigned long getFileSize(const char *fName);

/* Read the first 'length' bytes of the file into the buffer.
 * Returns the amount of bytes successfully read, or
 * -1 if opening the file fails. */
int readFirstPart(unsigned char *buff, unsigned int length,
                  const char *fileName);

/* Read the last 'length' bytes of the file into the buffer.
 * Returns the amount of bytes successfully read, or
 * -1 if opening the file or seeking to the end fails. */
int readLastPart(unsigned char *buff, int length,
                 const char *fileName);

/* Returns the index inside mem where the first occurrence
 * of query is found. Returns -1 if no match. */
long memfind(const char *mem,   unsigned long memSize,
             const char *query, unsigned long querySize);
